# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
from typing import Literal
from uuid import UUID

from more_itertools import one
from pydantic import Field
from pydantic import root_validator

from .._shared import AddressType
from .._shared import DictStrAny
from .._shared import EmployeeRef
from .._shared import EngagementRef
from .._shared import MOBase
from .._shared import OrganisationRef
from .._shared import OrgUnitRef
from .._shared import PersonRef
from .._shared import Validity
from .._shared import Visibility
from ._shared import Details


class AddressBase(MOBase):
    """
    A MO address object.
    """

    type_: str = Field("address", alias="type", description="The object type.")
    value: str = Field(description="Value of the address, e.g. street or phone number.")
    value2: str | None = Field(description="Optional second value of the address.")
    validity: Validity = Field(description="Validity of the address object.")


class AddressRead(AddressBase):
    """
    A MO address read object.
    Note that one and only one of {employee, org_unit, engagement} are given at any
    time.
    """

    address_type_uuid: UUID = Field(description="UUID of the address type klasse.")
    employee_uuid: UUID | None = Field(
        description="UUID of the employee related to the address."
    )
    org_unit_uuid: UUID | None = Field(
        description="UUID of the organisation unit related to the address."
    )
    engagement_uuid: UUID | None = Field(
        description="UUID of the engagement related to the address."
    )
    visibility_uuid: UUID | None = Field(
        description="UUID of the visibility klasse of the address."
    )

    # NOTE: The one and only one of {employee, org_unit, engagement} invariant
    # is not validated here because reads are assumed to originate from valid data.


class AddressWrite(AddressBase):
    """
    A MO address write object.
    Note that one and only one of {employee, org_unit, engagement} can be given.
    """

    address_type: AddressType = Field(
        description="Reference to the address type klasse."
    )
    employee: EmployeeRef | None = Field(
        description="Reference to the employee for which the address should be created."
    )
    org_unit: OrgUnitRef | None = Field(
        description=(
            "Reference to the organisation unit for which the address should "
            "be created."
        )
    )
    engagement: EngagementRef | None = Field(
        description=(
            "Reference to the engagement for which the address should be created."
        )
    )
    visibility: Visibility | None = Field(
        description="Reference to the Visibility klasse of the created address object."
    )

    # NOTE: This is not optimal handling of variability. In a perfect world,
    # we'd have an object_ref: Union[EmployeeRef, OrgUnitRef, EngagementRef]
    # field so that we do not have to check it like this.
    @root_validator
    def validate_references(cls, values: DictStrAny) -> DictStrAny:
        references = (
            values.get("employee"),
            values.get("org_unit"),
            values.get("engagement"),
        )
        too_short = ValueError("A reference must be specified")
        too_long = ValueError("Too many references specified.")
        one(
            filter(lambda ref: ref is not None, references),
            too_short=too_short,
            too_long=too_long,
        )

        return values


class AddressDetail(AddressWrite, Details):
    @root_validator
    def validate_references(cls, values: DictStrAny) -> DictStrAny:
        """Reference validator is overwritten because all refs are optional."""
        return values


class Address(MOBase):
    """
    A MO address object.
    """

    type_: Literal["address"] = Field(
        "address", alias="type", description="The object type."
    )
    value: str = Field(description="Value of the address, e.g. street or phone number.")
    value2: str | None = Field(description="Optional second value of the address.")
    address_type: AddressType = Field(
        description="Reference to the address type facet."
    )
    org: OrganisationRef | None = Field(
        description=(
            "Reference to the organisation under which the address should be created. "
            "MO only supports one organisation, so this is rarely used."
        )
    )
    person: PersonRef | None = Field(
        description=(
            "Reference to the person object for which the address should be created."
        )
    )
    org_unit: OrgUnitRef | None = Field(
        description=(
            "Reference to the organisation unit for which the address should be "
            "created."
        )
    )
    engagement: EngagementRef | None = Field(
        description=(
            "Reference to the engagement for which the address should be created."
        )
    )
    validity: Validity = Field(description="Validity of the created address object.")
    visibility: Visibility | None = Field(
        description="Reference to the Visibility klasse of the created address object."
    )

    @classmethod
    def from_simplified_fields(
        cls,
        value: str,
        address_type_uuid: UUID,
        from_date: str,
        uuid: UUID | None = None,
        to_date: str | None = None,
        value2: str | None = None,
        person_uuid: UUID | None = None,
        org_unit_uuid: UUID | None = None,
        engagement_uuid: UUID | None = None,
        visibility_uuid: UUID | None = None,
        org_uuid: UUID | None = None,
    ) -> "Address":
        """Create an address from simplified fields."""
        address_type = AddressType(uuid=address_type_uuid)
        org = OrganisationRef(uuid=org_uuid) if org_uuid else None
        validity = Validity(from_date=from_date, to_date=to_date)
        person = PersonRef(uuid=person_uuid) if person_uuid else None
        org_unit = OrgUnitRef(uuid=org_unit_uuid) if org_unit_uuid else None
        engagement = EngagementRef(uuid=engagement_uuid) if engagement_uuid else None
        visibility = Visibility(uuid=visibility_uuid) if visibility_uuid else None
        return cls(
            uuid=uuid,
            value=value,
            value2=value2,
            address_type=address_type,
            org=org,
            person=person,
            org_unit=org_unit,
            engagement=engagement,
            visibility=visibility,
            validity=validity,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ramodels', 'ramodels.lora', 'ramodels.mo', 'ramodels.mo.details']

package_data = \
{'': ['*']}

install_requires = \
['mkdocstrings[python]>=0.19.0,<0.20.0',
 'more-itertools>=9.0.0,<10.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0']

entry_points = \
{'console_scripts': ['lorafetch = ramodels.fetch:gen_models']}

setup_kwargs = {
    'name': 'ramodels',
    'version': '13.1.1',
    'description': 'Pydantic data models for OS2mo',
    'long_description': '<!--\nSPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>\nSPDX-License-Identifier: MPL-2.0\n-->\n\n\n# MoLoRa Data Models\n\nRAModels - MoLoRa data validation models powered by [pydantic](https://github.com/samuelcolvin/pydantic/#pydantic).\n\n## Versioning\nThis project uses [Semantic Versioning](https://semver.org/) with the following strategy:\n- MAJOR: Incompatible changes to existing data models\n- MINOR: Backwards compatible updates to existing data models OR new models added\n- PATCH: Backwards compatible bug fixes\n\n\n## Authors\n\nMagenta ApS <https://magenta.dk>\n\n## License\n- This project: [MPL-2.0](MPL-2.0.txt)\n- Dependencies:\n  - pydantic: [MIT](MIT.txt)\n\nThis project uses [REUSE](https://reuse.software) for licensing. All licenses can be found in the [LICENSES folder](LICENSES/) of the project.\n\n## Development\n### Prerequisites\n\n- [Poetry](https://github.com/python-poetry/poetry)\n- [Pre-commit](https://github.com/pre-commit/pre-commit)\n\n\n### Getting Started\n\n1. Clone the repository:\n`git clone git@git.magenta.dk:rammearkitektur/ra-data-models.git`\n\n2. Install all dependencies:\n`poetry install`\n\n3. Set up pre-commit:\n`pre-commit install`\n\n\n### Running the tests\n\nYou use `poetry` and `pytest` to run the tests:\n\n`poetry run pytest`\n\n\nYou can also run specific files\n\n`poetry run pytest tests/<test_folder>/<test_file.py>`\n\nand even use filtering with `-k`\n\n`poetry run pytest -k "Manager"`\n\n\nYou can use the flags `-vx` where `v` prints the test & `x` makes the test stop if any tests fails (Verbose, X-fail)\n\n### Pre-commit usage\nPre-commit must either be used via your virtual environment or globally.\nIf you want to pre-commit globally, the following extra dependencies are needed:\n`pip install mypy pydantic`\n\n\n### Models\n\n## LoRa\n`LoRa` implements the OIO standard version 1.1. The [standard](https://digitaliser.dk/resource/1569113) with\n[specification](https://www.digitaliser.dk/resource/1569113/artefact/Specifikationafserviceinterfacefororganisation-OIO-Godkendt%5bvs.1.1%5d.pdf?artefact=true&PID=1569586)\n',
    'author': 'Magenta',
    'author_email': 'info@magenta.dk',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://magenta.dk/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
